#!/bin/bash

CONFIG_VERSION="v1.0.3_d21.10.24"

# Получение исходных значений
BOARD_VERSION=$(repka-control get-board-version)
PINOUT=$(repka-control get-pinout)
MAXIMUM_FREQUENCY=$(repka-control get-freq)
CONSOLE_PARAM=$(repka-control get-console)
BOOT_MESSAGE=$(repka-control get-boot-message)
ROOT_DEVICE=$(lsblk -no pkname $(findmnt -n -o SOURCE /))

# Сохранение исходных значений для последующего сравнения и использования в функции check_need_reboot
BOARD_VERSION_SAVE=$BOARD_VERSION
PINOUT_SAVE=$PINOUT
MAXIMUM_FREQUENCY_SAVE=$MAXIMUM_FREQUENCY
CONSOLE_PARAM_SAVE=$CONSOLE_PARAM
BOOT_MESSAGE_SAVE=$BOOT_MESSAGE

check_need_reboot() {
  IS_SAME=true

  [ "$BOARD_VERSION" != "$BOARD_VERSION_SAVE" ] && IS_SAME=false
  [ "$PINOUT" != "$PINOUT_SAVE" ] && IS_SAME=false
  [ "$MAXIMUM_FREQUENCY" != "$MAXIMUM_FREQUENCY_SAVE" ] && IS_SAME=false

  if [ "$IS_SAME" = false ]; then
    if (
          whiptail \
            --title "Необходима перезагрузка" \
            --yesno "Вы внесли изменения в конфигурацию Repka Pi 4\nДля вступления изменений в силу необходимо перезагрузить устройство\nВыполнить перезагрузку?" \
            10 80) then
            echo "Перезагружаем $?."
            reboot
      else
       return 0
      fi
  fi
}

need_reboot() {
  if (
    whiptail \
      --title "Необходима перезагрузка" \
      --yesno "Чтобы изменения вступили в силу необходимо перезагрузить Repka Pi 4\nВыполнить перезагрузку?" \
      10 80) then
        echo "Перезагружаем $?."
        reboot
  else
   return 0
  fi
}

# =================== 3 Interface Options ===================

set_pinout() {
  PINOUT=$1
  repka-control set-pinout "$1"
}

set_console_param() {
  CONSOLE_PARAM=$1
  repka-control set-console "$1"
}

switch_off_boot_message() {
  BOOT_MESSAGE="0"
  repka-control set-boot-message "0"
  dd if=/boot/spl/u-boot-sunxi-with-spl-silent.bin of=/dev/${ROOT_DEVICE} bs=1024 seek=8 conv=notrunc
}

switch_on_boot_message() {
  BOOT_MESSAGE="1"
  repka-control set-boot-message "1"
  dd if=/boot/spl/u-boot-sunxi-with-spl.bin of=/dev/${ROOT_DEVICE} bs=1024 seek=8 conv=notrunc
}

set_interface() {
  repka-control set-custom-port "$1" "$2"
  if [ $PINOUT = "custom" ]; then
      set_pinout "custom" && need_reboot
  fi
}

  select_interface() {
  if [ "$#" -ne 1 ]; then
    echo "Необходимо использовать аргумент CP с номером"
    return 1
  fi

  case "$1" in
    CP1)
      CP1_RESULT=$(repka-control get-custom-port 1)
      CP1_STATUS=$([[ "${CP1_RESULT}" = "" ]] && echo "ON" || echo "OFF")
      CP2_STATUS=$([[ "${CP1_RESULT}" = "i2c1" ]] && echo "ON" || echo "OFF")

      RESULT=$(
        whiptail \
          --title "CP1 | 3,5 Pins" \
          --radiolist \
          "Выберите доступный интерфейс:" \
          --cancel-button "Назад/Back" \
          --ok-button "Выбрать/Select" \
          20 50 4 \
          "GPIO" "" ${CP1_STATUS} \
          "I2C1" "" ${CP2_STATUS} \
          3>&1 1>&2 2>&3)

      RET=$?
      if [ $RET -eq 1 ] || [ $RET -eq 255 ]; then
        custom_pinout_config "3 Interface Options/I3 Custom Pinout"
      elif [ $RET -eq 0 ]; then
        case "$RESULT" in
          "GPIO") set_interface "1" "" && custom_pinout_config "3 Interface Options/I3 Custom Pinout" ;;
          "I2C1") set_interface "1" "i2c1" && custom_pinout_config "3 Interface Options/I3 Custom Pinout" ;;
          *) whiptail --msgbox "Ошибка" 20 60 1 || return 0;;
        esac || whiptail --msgbox "Что то тут не так $RESULT" 20 60 1 || return 0
      fi
      ;;

    CP2)
      CP2_RESULT=$(repka-control get-custom-port 2)
      CP1_STATUS=$([[ "${CP2_RESULT}" = "" ]] && echo "ON" || echo "OFF")
      CP2_STATUS=$([[ "${CP2_RESULT}" = "spi0" ]] && echo "ON" || echo "OFF")
      CP3_STATUS=$([[ "${CP3_RESULT}" = "i2c3" ]] && echo "ON" || echo "OFF")

      RESULT=$(
        whiptail \
          --title "CP2 | 19, 21, 23, 24, 26 Pins" \
          --radiolist \
          "Выберите доступный интерфейс:" \
          --cancel-button "Назад/Back" \
          --ok-button "Выбрать/Select" \
          20 50 4 \
          "GPIO" "" ${CP1_STATUS} \
          "SPI0" "" ${CP2_STATUS} \
          "I2C3" "" ${CP3_STATUS} \
          3>&1 1>&2 2>&3)

      RET=$?
      if [ $RET -eq 1 ] || [ $RET -eq 255 ]; then
        custom_pinout_config "3 Interface Options/I3 Custom Pinout"
      elif [ $RET -eq 0 ]; then
        case "$RESULT" in
          "GPIO") set_interface "2" "" && custom_pinout_config "3 Interface Options/I3 Custom Pinout" ;;
          "SPI0") set_interface "2" "spi0" && custom_pinout_config "3 Interface Options/I3 Custom Pinout" ;;
          "I2C3") set_interface "3" "i2c3" && custom_pinout_config "3 Interface Options/I3 Custom Pinout" ;;
          *) whiptail --msgbox "Ошибка" 20 60 1 || return 0;;
        esac || whiptail --msgbox "Что то тут не так $RESULT" 20 60 1 || return 0
      fi
      ;;

    CP3)
      CP2_RESULT=$(repka-control get-custom-port 3)
      CP1_STATUS=$([[ "${CP2_RESULT}" = "" ]] && echo "ON" || echo "OFF")
      CP2_STATUS=$([[ "${CP2_RESULT}" = "i2c2" ]] && echo "ON" || echo "OFF")
      CP3_STATUS=$([[ "${CP2_RESULT}" = "uart3" ]] && echo "ON" || echo "OFF")

      RESULT=$(
        whiptail \
          --title "CP3 | 27, 28 Pins" \
          --radiolist \
          "Выберите доступный интерфейс:" \
          --cancel-button "Назад/Back" \
          --ok-button "Выбрать/Select" \
          20 50 4 \
          "GPIO" "" ${CP1_STATUS} \
          "I2C2" "" ${CP2_STATUS} \
          "UART3" "" ${CP3_STATUS} \
          3>&1 1>&2 2>&3)

      RET=$?
      if [ $RET -eq 1 ] || [ $RET -eq 255 ]; then
        custom_pinout_config "3 Interface Options/I3 Custom Pinout"
      elif [ $RET -eq 0 ]; then
        case "$RESULT" in
          "GPIO") set_interface "3" "" && custom_pinout_config "3 Interface Options/I3 Custom Pinout" ;;
          "I2C2") set_interface "3" "i2c2" && custom_pinout_config "3 Interface Options/I3 Custom Pinout" ;;
          "UART3") set_interface "3" "uart3" && custom_pinout_config "3 Interface Options/I3 Custom Pinout" ;;
          *) whiptail --msgbox "Ошибка" 20 60 1 || return 0;;
        esac || whiptail --msgbox "Что то тут не так $RESULT" 20 60 1 || return 0
      fi
      ;;

    CP4)
      CP2_RESULT=$(repka-control get-custom-port 4)
      CP1_STATUS=$([[ "${CP2_RESULT}" = "" ]] && echo "ON" || echo "OFF")
      CP2_STATUS=$([[ "${CP2_RESULT}" = "s_uart" ]] && echo "ON" || echo "OFF")

      RESULT=$(
        whiptail \
          --title "CP3 | 27, 28 Pins" \
          --radiolist \
          "Выберите доступный интерфейс:" \
          --cancel-button "Назад/Back" \
          --ok-button "Выбрать/Select" \
          20 50 4 \
          "GPIO" "" ${CP1_STATUS} \
          "S-UART" "" ${CP2_STATUS} \
          3>&1 1>&2 2>&3)

      RET=$?
      if [ $RET -eq 1 ] || [ $RET -eq 255 ]; then
        custom_pinout_config "3 Interface Options/I3 Custom Pinout"
      elif [ $RET -eq 0 ]; then
        case "$RESULT" in
          "GPIO") set_interface "4" "" && custom_pinout_config "3 Interface Options/I3 Custom Pinout" ;;
          "S-UART") set_interface "4" "s_uart" && custom_pinout_config "3 Interface Options/I3 Custom Pinout" ;;
          *) whiptail --msgbox "Ошибка" 20 60 1 || return 0;;
        esac || whiptail --msgbox "Что то тут не так $RESULT" 20 60 1 || return 0
      fi
      ;;

    CP5)
      CP2_RESULT=$(repka-control get-custom-port 5)
      CP1_STATUS=$([[ "${CP2_RESULT}" = "" ]] && echo "ON" || echo "OFF")
      CP2_STATUS=$([[ "${CP2_RESULT}" = "w1_gpio" ]] && echo "ON" || echo "OFF")

      RESULT=$(
        whiptail \
          --title "CP5 | 7 Pin" \
          --radiolist \
          "Выберите доступный интерфейс:" \
          --cancel-button "Назад/Back" \
          --ok-button "Выбрать/Select" \
          20 50 4 \
          "GPIO" "" ${CP1_STATUS} \
          "W1-GPIO" "" ${CP2_STATUS} \
          3>&1 1>&2 2>&3)

      RET=$?
      if [ $RET -eq 1 ] || [ $RET -eq 255 ]; then
        custom_pinout_config "3 Interface Options/I3 Custom Pinout"
      elif [ $RET -eq 0 ]; then
        case "$RESULT" in
          "GPIO") set_interface "5" "" && custom_pinout_config "3 Interface Options/I3 Custom Pinout" ;;
          "W1-GPIO") set_interface "5" "w1_gpio" && custom_pinout_config "3 Interface Options/I3 Custom Pinout" ;;
          *) whiptail --msgbox "Ошибка" 20 60 1 || return 0;;
        esac || whiptail --msgbox "Что то тут не так $RESULT" 20 60 1 || return 0
      fi
      ;;

    *)
        echo "Invalid argument: $1"
        return 1
        ;;
  esac
}

sd2emmc() {
  nand-sata-install && return
}

boot_messages() {
  BREADCRUBS=$1
  BM1_STATUS=$([[ ${BOOT_MESSAGE} = "1" ]] && echo "ON" || echo "OFF")
  BM2_STATUS=$([[ ${BOOT_MESSAGE} = "0" ]] && echo "ON" || echo "OFF")

  RESULT=$(
    whiptail \
      --title "Repka Pi утилита для настройки (repka-config) ${VERSION}" \
      --radiolist \
      --cancel-button "Назад/Back" \
      --ok-button "Выбрать/Select" \
      "$BREADCRUBS" \
      20 100 10 \
      "BM1 | Включить вывод сообщений загрузчика U-Boot" "" $BM1_STATUS \
      "BM2 | Отключить вывод сообщений загрузчика U-Boot" "" $BM2_STATUS \
      3>&1 1>&2 2>&3)

  RET=$?
  if [ $RET -eq 1 ] || [ $RET -eq 255 ]; then
    interface_options "3 Interface Options"
  elif [ $RET -eq 0 ]; then
    case "$RESULT" in
      BM1\ *) switch_on_boot_message && need_reboot ;;
      BM2\ *) switch_off_boot_message && need_reboot ;;
      *) whiptail --msgbox "Ошибка" 20 60 1 || return 0;;
    esac || whiptail --msgbox "Что то тут не так $RESULT" 20 60 1 || return 0
  fi
}

console_param() {
  BREADCRUBS=$1
    CP1_STATUS=$([[ ${CONSOLE_PARAM} = "both" ]] && echo "ON" || echo "OFF")
    CP2_STATUS=$([[ ${CONSOLE_PARAM} = "display" ]] && echo "ON" || echo "OFF")
    CP3_STATUS=$([[ ${CONSOLE_PARAM} = "serial" ]] && echo "ON" || echo "OFF")

    RESULT=$(
      whiptail \
        --title "Repka Pi утилита для настройки (repka-config) ${VERSION}" \
        --radiolist \
        --cancel-button "Назад/Back" \
        --ok-button "Выбрать/Select" \
        "$BREADCRUBS" \
        20 100 10 \
        "CP1 | Дисплей (tty6) и UART0 " "" $CP1_STATUS \
        "CP2 | Дисплей (tty6) " "" $CP2_STATUS \
        "CP3 | UART0 " "" $CP3_STATUS \
        3>&1 1>&2 2>&3)

    RET=$?
    if [ $RET -eq 1 ] || [ $RET -eq 255 ]; then
      interface_options "3 Interface Options"
    elif [ $RET -eq 0 ]; then
      case "$RESULT" in
        CP1\ *)
          set_console_param "both"
          need_reboot
          interface_options "3 Interface Options" ;;

        CP2\ *)
          set_console_param "display"
          need_reboot
          interface_options "3 Interface Options" ;;

        CP3\ *)
          set_console_param "serial"
          need_reboot
          interface_options "3 Interface Options" ;;

        *) whiptail --msgbox "Ошибка" 20 60 1 || return 0;;
      esac || whiptail --msgbox "Что то тут не так $RESULT" 20 60 1 || return 0
    fi
}

custom_pinout_config() {
  BREADCRUBS=$1
  CP1="$(repka-control get-custom-port 1)"
  CP2="$(repka-control get-custom-port 2)"
  CP3="$(repka-control get-custom-port 3)"
  CP4="$(repka-control get-custom-port 4)"
  CP5="$(repka-control get-custom-port 5)"

  RESULT=$(
    whiptail \
      --title "Repka Pi утилита для настройки (repka-config) ${VERSION}"  \
      --menu "$BREADCRUBS \n\nВыберите нужный набор пинов для установки конкретного интерфейса:" \
      20 80 10 \
      --cancel-button "Выйти/Finish" \
      --ok-button "Выбрать/Select" \
      "3, 5 Pins" "  - $(if [ -z "$CP1" ]; then echo '[GPIO-Выбран]'; else echo 'GPIO'; fi)/$(if [ $CP1 = "i2c1" ]; then echo '[I2C1-Выбран]'; else echo 'I2C1'; fi)" \
      "7 Pin" "  - $(if [ -z "$CP5" ]; then echo '[GPIO-Выбран]'; else echo 'GPIO'; fi)/$(if [ $CP5 = "w1_gpio" ]; then echo '[W1-GPIO-Выбран]'; else echo 'W1-GPIO'; fi)" \
      "16, 18 Pins" "  - $(if [ -z "$CP4" ]; then echo '[GPIO-Выбран]'; else echo 'GPIO'; fi)/$(if [ $CP4 = "s_uart" ]; then echo '[S-UART-Выбран]'; else echo 'S-UART'; fi)" \
      "19, 21, 23, 24, 26 Pins" "  - $(if [ -z "$CP2" ]; then echo '[GPIO-Выбран]'; else echo 'GPIO'; fi)/$(if [ $CP2 = "spi0" ]; then echo '[SPI0-Выбран]'; else echo 'SPI0'; fi)" \
      "27, 28 Pins" "  - $(if [ -z "$CP3" ]; then echo '[GPIO-Выбран]'; else echo 'GPIO'; fi)/$(if [ $CP3 = "i2c2" ]; then echo '[I2C2-Выбран]'; else echo 'I2C2'; fi)/$(if [ $CP3 = "uart3" ]; then echo '[UART3-Выбран]'; else echo 'UART3'; fi)" \
      3>&1 1>&2 2>&3)

  RET=$?
  if [ $RET -eq 1 ] || [ $RET -eq 255 ]; then
    interface_options "3 Interface Options"
  elif [ $RET -eq 0 ]; then
    case "$RESULT" in
      "3, 5 Pins") select_interface "CP1" ;;
      "7 Pin") select_interface "CP5" ;;
      "16, 18 Pins") select_interface "CP4" ;;
      "19, 21, 23, 24, 26 Pins") select_interface "CP2" ;;
      "27, 28 Pins") select_interface "CP3" ;;
      *) whiptail --msgbox "Ошибка" 20 60 1 ;;
    esac || whiptail --msgbox "Что то тут не так $RESULT" 20 60 1
  fi
}

# =================== 7 Update ===================

repka_os_update() {
  REPKA_OS_UPDATE=/usr/local/repka-os-update
  wget -O $REPKA_OS_UPDATE --timeout=120 https://download.repka-pi.ru/repka-pi4/repka_os_update/repka-os-update
  chmod +x $REPKA_OS_UPDATE
  $REPKA_OS_UPDATE
  rm $REPKA_OS_UPDATE
}

ubuntu_update() {
  apt-get update &&
  apt-get upgrade &&
  printf "Ожидаем 5 секунд до повторного запуска repka-config\n" &&
  sleep 5 &&
  exec repka-config
}

# I2 Pinout Options
pinout_options() {
  BREADCRUBS=$1

  PINOUT_1=$([[ "${PINOUT}" = "1" ]] && echo "ON" || echo "OFF")
  PINOUT_2=$([[ "${PINOUT}" = "2" ]] && echo "ON" || echo "OFF")
  PINOUT_3=$([[ "${PINOUT}" = "3" ]] && echo "ON" || echo "OFF")
  PINOUT_4=$([[ "${PINOUT}" = "4" ]] && echo "ON" || echo "OFF")
  PINOUT_5=$([[ "${PINOUT}" = "5" ]] && echo "ON" || echo "OFF")
  PINOUT_CUSTOM=$([[ "${PINOUT}" = "custom" ]] && echo "ON" || echo "OFF")

  OPTION=$(
    whiptail \
      --title "Выбор распиновки" \
      --radiolist "$BREADCRUBS \n\nВыберите вариант распиновки портов Repka Pi:" \
      20 85 10 \
      --cancel-button "Назад/Back" \
      --ok-button "Выбрать/Select"\
      "1 Вариант" "  - GPIO-24, UART-1, PWM-1 (По умолчанию) " $PINOUT_1 \
      "2 Вариант" "  - GPIO-18, UART-1, SPI-1, I2C-1, PWM-1" $PINOUT_2 \
      "3 Вариант" "  - GPIO-16, UART-1, SPI-1, I2C-2, PWM-1" $PINOUT_3 \
      "4 Вариант" "  - GPIO-14, UART-2, S-UART-1, SPI-1, I2C-1, PWM-1" $PINOUT_4 \
      "9 Вариант" "  - GPIO-22, UART-1, I2C-1, W1_GPIO-1" $PINOUT_5 \
      "10 Свой вариант" " - Выберите необходимые для вас интерфейсы" $PINOUT_CUSTOM \
      3>&1 1>&2 2>&3)

  RET=$?
  if [ $RET -eq 1 ] || [ $RET -eq 255 ]; then
    interface_options "3 Interface Options"
  elif [ $RET -eq 0 ]; then
    case "$OPTION" in
  1\ *) set_pinout 1 && need_reboot ;;
  2\ *) set_pinout 2 && need_reboot ;;
  3\ *) set_pinout 3 && need_reboot ;;
  4\ *) set_pinout 4 && need_reboot ;;
  9\ *) set_pinout 5 && need_reboot ;;
  10\ *) set_pinout "custom" && need_reboot ;;
      *) whiptail --msgbox "Ошибка" 20 60 1 ;;
    esac || whiptail --msgbox "Что то тут не так $RESULT" 20 60 1
  fi
}

# 1 System Options
system_options() {
  BREADCRUBS="$1/"
  RESULT=$(
    whiptail \
      --title "Repka Pi утилита для настройки (repka-config) ${VERSION}" \
      --menu "$BREADCRUBS" \
      20 80 10 \
      --cancel-button "Назад/Back"\
      --ok-button "Выбрать/Select" \
  	  "S1 OS Copying " "Копирование ОС с одного накопителя на другой" \
      3>&1 1>&2 2>&3)

    RET=$?

    if [ $RET -eq 1 ] || [ $RET -eq 255 ]; then
      return 0
    elif [ $RET -eq 0 ]; then
      case "$RESULT" in
        S1\ *) sd2emmc ;;
        *) whiptail --msgbox "Ошибка" 20 60 1 || return 0;;
      esac || whiptail --msgbox "Что то тут не так $RESULT" 20 60 1 || return 0
    fi
}

# 3 Interface Options
interface_options() {
  BREADCRUBS="$1/"
  RESULT=$(
    whiptail \
    --title "Repka Pi утилита для настройки (repka-config) ${VERSION}" \
    --menu "$BREADCRUBS" \
    20 80 10 \
    --cancel-button "Назад/Back" \
    --ok-button "Выбрать/Select" \
  	"I1 Console Param " "Настройка параметров консоли" \
  	"I2 Pinout Options" "Выбор распиновки" \
  	"I3 Custom Pinout" "Настройка своего варианта распиновки" \
  	"I4 Boot Message " "Выводить ли сообщения загрузчика U-Boot" \
    3>&1 1>&2 2>&3)

    RET=$?

    if [ $RET -eq 1 ] || [ $RET -eq 255 ]; then
      return 0
    elif [ $RET -eq 0 ]; then
      case "$RESULT" in
        I1\ *) console_param "${BREADCRUBS}I1 Console Param " ;;
        I2\ *) pinout_options "${BREADCRUBS}I2 Pinout Options" ;;
        I3\ *) custom_pinout_config "${BREADCRUBS}I3 Custom Pinout" ;;
        I4\ *) boot_messages "${BREADCRUBS}I4 Boot Message" ;;
        *) whiptail --msgbox "В разработке $RESULT" 20 60 1 ;;
      esac || whiptail --msgbox "Что то тут не так $RESULT" 20 60 1 || return 0
    fi
}


hardware_acceleration_turn_on() {
  apt install -y ffmpeg-v4l2request mpv \
  && echo -e "hwdec=drm\ndrm-drmprime-video-plane=primary\ndrm-draw-plane=overlay" | tee /etc/mpv/mpv.conf \
  && echo 1 > /etc/repka-wallpaper/hardware-acceleration \
  && repka-wallpaper \
  && need_reboot
}

hardware_acceleration_turn_off() {
  apt purge -y ffmpeg-v4l2request mpv \
    && apt autoremove -y \
    && echo 0 > /etc/repka-wallpaper/hardware-acceleration \
    && repka-wallpaper \
    && need_reboot
}

hardware_acceleration() {
    BREADCRUBS="$1/"
    RESULT=$(
      whiptail \
        --title "Repka Pi утилита для настройки (repka-config) ${VERSION}" \
        --menu "$BREADCRUBS" \
        20 80 10 \
        --cancel-button "Назад/Back" \
        --ok-button "Выбрать/Select" \
    	  "Turn-On " "Включить аппаратное ускорение" \
    	  "Turn-Off " "Выключить аппаратное ускорение" \
        3>&1 1>&2 2>&3)

      RET=$?
      if [ $RET -eq 1 ] || [ $RET -eq 255 ]; then
        return 0
      elif [ $RET -eq 0 ]; then
        case "$RESULT" in
          Turn-On\ *) hardware_acceleration_turn_on ;;
          Turn-Off\ *) hardware_acceleration_turn_off ;;
          *) whiptail --msgbox "Ошибка" 20 60 1 || return 0;;
        esac || whiptail --msgbox "Что то тут не так $RESULT" 20 60 1 || return 0
      fi
}

# 4 Performance Options
performance_options() {
  BREADCRUBS="$1/"
  RESULT=$(
    whiptail \
      --title "Repka Pi утилита для настройки (repka-config) ${VERSION}" \
      --menu "$BREADCRUBS" \
      20 80 10 \
      --cancel-button "Назад/Back" \
      --ok-button "Выбрать/Select" \
  	  "P1 Hardware Acceleration " "Аппаратное ускорение" \
      3>&1 1>&2 2>&3)

    RET=$?
    if [ $RET -eq 1 ] || [ $RET -eq 255 ]; then
      return 0
    elif [ $RET -eq 0 ]; then
      case "$RESULT" in
        P1\ *) hardware_acceleration "${BREADCRUBS}P1 Hardware Acceleration";;
        *) whiptail --msgbox "Ошибка" 20 60 1 || return 0;;
      esac || whiptail --msgbox "Что то тут не так $RESULT" 20 60 1 || return 0
    fi
}

# 7 Update
update() {
  BREADCRUBS="$1/"
  echo " А есть ли интернет....хм... ? "
  if ping -c 1 repka-pi.ru > /dev/null 2>&1; then
    echo " ура интернет ^___^ "

    RESULT=$(
      whiptail \
        --title "Repka Pi утилита для настройки (repka-config) ${VERSION}" \
        --menu "$BREADCRUBS" \
        20 80 10 \
        --cancel-button "Назад/Back" \
        --ok-button "Выбрать/Select" \
        "U1 Ubuntu update pkg " "Обновить пакеты Ubuntu" \
        "U2 Repka OS update " "Обновить файлы Repka-OS" \
        3>&1 1>&2 2>&3)

    RET=$?

    if [ $RET -eq 1 ] || [ $RET -eq 255 ]; then
      return 0
    elif [ $RET -eq 0 ]; then
      case "$RESULT" in
        U1\ *) ubuntu_update ;;
        U2\ *) repka_os_update ;;
        *) whiptail --msgbox "Ошибка" 20 60 1 || return 0;;
      esac || whiptail --msgbox "Что то тут не так $RESULT" 20 60 1 || return 0
    fi
  else
    whiptail --msgbox "Отсутствует подключение к интернету  " 10 40 1 || return 0
  fi
}

# Основной цикл утилиты настройки с главным меню
while true; do
  RESULT=$(
    whiptail \
      --title "Repka Pi утилита для настройки (repka-config) ${CONFIG_VERSION}"  \
      --menu "Выберите настраиваемую опцию. (Для выхода из repka-config нажмите ESC)" \
      20 80 10 \
      --cancel-button "Выйти/Finish" \
      --ok-button "Выбрать/Select" \
      "1 System Options" "Системные настройки ОС" \
      "2 Display Options" "Настройка параметров дисплея" \
      "3 Interface Options" "Настройка подключения интерфейсов" \
      "4 Performance Options" "Настройка производительности" \
      "5 Localization Options" "Настройка региона и языка" \
      "6 Advanced Options" "Продвинутые настройки Repka OS" \
      "7 Update" "Обновление компонентов Repka OS" \
      "8 Board Options" "Параметры платы Repka Pi" \
      "9 About repka-config" "Информация по утилите настройки" \
      3>&1 1>&2 2>&3)

  RET=$?
  if [ $RET -eq 1 ]; then
    exit
    return 0
  elif [ $RET -eq 0 ]; then
    case "$RESULT" in
      1\ *) system_options "1 System Options" ;;
      3\ *) interface_options "3 Interface Options" ;;
      4\ *) performance_options "4 Performance Options" ;;
      7\ *) update "7 Update" ;;
      *) whiptail --msgbox "В разработке $RESULT" 20 60 1 ;;
    esac || whiptail --msgbox "В разработке $RESULT" 20 60 1
  else
    check_need_reboot
    exit 1
  fi
done